Index: src/icestypes.h
===================================================================
RCS file: /cvsroot/ices/src/icestypes.h,v
retrieving revision 1.4
diff -u -p -r1.4 icestypes.h
--- src/icestypes.h	2000/08/24 06:18:40	1.4
+++ src/icestypes.h	2001/06/10 16:39:23
@@ -24,33 +24,47 @@
 typedef enum {icy_header_protocol_e = 0, xaudiocast_header_protocol_e = 1} header_protocol_t;
 typedef enum {ices_playlist_python_e = 0, ices_playlist_builtin_e = 1, ices_playlist_perl_e = 2} playlist_type_t;
 
+typedef struct ices_stream_config_St {
+  shout_conn_t conn;
+  void* encoder_state;
+  int encoder_initialised;
+
+  char* mount;
+
+  char* name;
+  char* genre;
+  char* description;
+  char* url;
+  int ispublic;
+
+  int reencode;
+  int bitrate;
+  int out_samplerate;
+  int out_numchannels;
+
+  struct ices_stream_config_St* next;
+} ices_stream_config_t;
+
 typedef struct ices_config_St {
-	char *host;
-	int port;
-	char *mount;
-	char *password;
-	header_protocol_t header_protocol;
-	char *name;
-	char *genre;
-	char *description;
-	char *url;
-	int bitrate;
-	int ispublic;
-	int daemon;
-	int randomize_playlist;
-	int pre_dj;
-	int post_dj;
-	int playlist_type;
-	int verbose;
-	int reencode;
-	int out_samplerate;
-	int out_numchannels;
-	char *dumpfile;
-	char *configfile;
-	char *playlist_file;
-	char *interpreter_file;
-	char *base_directory;
-	FILE *logfile;
+  char *host;
+  int port;
+  char *password;
+  header_protocol_t header_protocol;
+  int daemon;
+  int randomize_playlist;
+  int pre_dj;
+  int post_dj;
+  int playlist_type;
+  int verbose;
+  int reencode;
+  char *dumpfile;
+  char *configfile;
+  char *playlist_file;
+  char *interpreter_file;
+  char *base_directory;
+  FILE *logfile;
+
+  ices_stream_config_t* streams;
 }ices_config_t;
 
 #endif
Index: src/id3.c
===================================================================
RCS file: /cvsroot/ices/src/id3.c,v
retrieving revision 1.12
diff -u -p -r1.12 id3.c
--- src/id3.c	2001/06/08 19:49:42	1.12
+++ src/id3.c	2001/06/10 16:39:24
@@ -22,7 +22,6 @@
 
 #include <thread.h>
 
-extern shout_conn_t conn;
 extern ices_config_t ices_config;
 
 static char *ices_id3_filename = NULL;
@@ -183,33 +182,36 @@ ices_id3_update_metadata (const char *fi
 void *
 ices_id3_update_thread (void *arg)
 {
-	int ret;
-	char metastring[1024], song[2048], artistspace[1024], titlespace[1024],
-	     filespace[1024];
-	char *id3_artist = ices_id3_get_artist (artistspace, 1024);
-	char *id3_song = ices_id3_get_title (titlespace, 1024);
-	const char *filename = ices_id3_get_filename (filespace, 1024);
+  ices_stream_config_t* stream;
+  int ret;
+  char metastring[1024], song[2048], artistspace[1024], titlespace[1024],
+    filespace[1024];
+  char *id3_artist = ices_id3_get_artist (artistspace, 1024);
+  char *id3_song = ices_id3_get_title (titlespace, 1024);
+  const char *filename = ices_id3_get_filename (filespace, 1024);
+	
+  if (id3_artist) {
+    sprintf (song, "%s - %s", id3_artist, id3_song ? id3_song : filename);
+  } else {
+    sprintf (song, "%s", (id3_song != NULL) ? id3_song : ices_id3_filename_cleanup (filename, metastring, 1024));
+  }
+	
+  if (ices_config.header_protocol == icy_header_protocol_e)
+    sprintf (metastring, "%s", song);
+  else
+    sprintf (metastring, "%s", song); /* This should have length as well but libshout doesn't handle it correctly */
+
+  for (stream = ices_config.streams; stream; stream = stream->next) {
+    ret = shout_update_metadata (&stream->conn, metastring);
+	
+    if (ret != 1)
+      ices_log ("Updating metadata on server failed.");
+    else
+      ices_log ("Updated metadata on server to: %s", song);
+  }
 	
-	if (id3_artist) {
-		sprintf (song, "%s - %s", id3_artist, id3_song ? id3_song : filename);
-	} else {
-		sprintf (song, "%s", (id3_song != NULL) ? id3_song : ices_id3_filename_cleanup (filename, metastring, 1024));
-	}
-	
-	if (ices_config.header_protocol == icy_header_protocol_e)
-		sprintf (metastring, "%s", song);
-	else
-		sprintf (metastring, "%s", song); /* This should have length as well but libshout doesn't handle it correctly */
-
-	ret = shout_update_metadata (&conn, metastring);
-	
-	if (ret != 1)
-		ices_log ("Updating metadata on server failed.");
-	else
-		ices_log ("Updated metadata on server to: %s", song);
-	
-	thread_exit (0);
-	return 0;
+  thread_exit (0);
+  return 0;
 }
 
 /* Function that does the id3 tag parsing of a file */
Index: src/reencode.c
===================================================================
RCS file: /cvsroot/ices/src/reencode.c,v
retrieving revision 1.8
diff -u -p -r1.8 reencode.c
--- src/reencode.c	2001/06/10 16:07:00	1.8
+++ src/reencode.c	2001/06/10 16:39:24
@@ -31,8 +31,7 @@
 
 extern ices_config_t ices_config;
 
-static int ices_reencode_initialized = 0;
-static lame_global_flags* ices_reencode_flags;
+static int nsamples; /* per call to lame_decode */
 static short int right[4096 * 30]; /* Probably overkill like hell, can someone calculate a better value please? */
 static short int left[4096 * 30];
 
@@ -44,47 +43,42 @@ static short int left[4096 * 30];
 void
 ices_reencode_initialize (void)
 {
-	if (!ices_config.reencode) 
-		return;
+  ices_stream_config_t* stream;
+  lame_global_flags* lame;
 
-	if (! (ices_reencode_flags = lame_init ())) {
-		ices_log ("Error: initialization of liblame failed!");
-		ices_setup_shutdown ();
-	}
-
-	lame_set_brate (ices_reencode_flags, ices_config.bitrate);
-	if (ices_config.out_numchannels != -1)
-		lame_set_num_channels (ices_reencode_flags, ices_config.out_numchannels);
-	if (ices_config.out_samplerate != -1)
-		lame_set_out_samplerate (ices_reencode_flags, ices_config.out_samplerate);
-
-	if (lame_init_params (ices_reencode_flags) == -1) {
-		ices_log ("Error: lame_init_params() failed!");
-	}
+  /* are any streams reencoding? */
+  for (stream = ices_config.streams; stream; stream = stream->next)
+    if (stream->reencode)
+      ices_config.reencode = 1;
+
+  if (! ices_config.reencode)
+    return;
+
+  for (stream = ices_config.streams; stream; stream = stream->next) {
+    if (! (stream->encoder_state = lame_init())) {
+      ices_log ("Error: initialization of liblame failed!");
+      ices_setup_shutdown ();
+    }
+
+    lame = (lame_global_flags*) stream->encoder_state;
+
+    lame_set_brate (lame, stream->bitrate);
+    if (stream->out_numchannels > 0)
+      lame_set_num_channels (lame, stream->out_numchannels);
+    if (stream->out_samplerate > 0)
+      lame_set_out_samplerate (lame, stream->out_samplerate);
+
+    lame_set_original (lame, 0);
+
+    if (lame_init_params (lame) < 0) {
+      ices_log ("Error: Second stage init of lame failed!");
+      ices_setup_shutdown ();
+    }
 
-	ices_log_debug ("Initializing lame version %s\n", get_lame_version ());
-	ices_reencode_initialized = 1;
-}
-
-/* Tell the reencoder what number of channels the current song has */
-void
-ices_reencode_set_channels (int channels)
-{
-	ices_reencode_flags->num_channels = channels;
-}
-
-/* Tell the reencoder what sample rate the current song is */
-void
-ices_reencode_set_sample_rate (int samplerate)
-{
-	ices_reencode_flags->in_samplerate = samplerate;
-}
+    stream->encoder_initialised = 1;
+  }
 
-/* What mode is the current song in? */
-void
-ices_reencode_set_mode (int mode)
-{
-	ices_reencode_flags->mode = mode;
+  ices_log_debug ("Using lame version %s\n", get_lame_version ());
 }
 
 /* For each song, reset the liblame engine, otherwize it craps out if
@@ -92,36 +86,56 @@ ices_reencode_set_mode (int mode)
 void
 ices_reencode_reset (void) 
 {
-	if (lame_decode_init () == -1) {
-		ices_log ("Error: initialization of liblame's decoder failed!");
-		ices_setup_shutdown ();
-	}
+  if (lame_decode_init () < 0) {
+    ices_log ("Error: initialization of liblame's decoder failed!");
+    ices_setup_shutdown ();
+  }
 }
 
 /* If initialized, shutdown the reencoding engine */
 void
 ices_reencode_shutdown (void)
 {
-  if (ices_reencode_initialized)
-    lame_close (ices_reencode_flags);
+  ices_stream_config_t* stream;
+
+  for (stream = ices_config.streams; stream; stream = stream->next)
+    if (stream->encoder_initialised)
+      lame_close ((lame_global_flags*) stream->encoder_state);
 }
 
+/* decode buffer, of length buflen, into left and right. Stream-independent
+ * (do this once per chunk, not per stream). Result is number of samples
+ * for ices_reencode_reencode_chunk. */
+int
+ices_reencode_decode (unsigned char* buf, size_t blen)
+{
+  return (nsamples = lame_decode (buf, blen, left, right));
+}
+
 /* reencode buff, of len buflen, put max outlen reencoded bytes in outbuf */
 int
-ices_reencode_reencode_chunk (unsigned char *buff, int buflen, unsigned char *outbuf, int outlen)
+ices_reencode_reencode_chunk (ices_stream_config_t* stream, unsigned char *buff,
+			      int buflen, unsigned char *outbuf, int outlen)
 {
-	int decode_ret = lame_decode ((char *)buff, buflen, left, right);
+  lame_global_flags* lame = (lame_global_flags*) stream->encoder_state;
 
-	if (decode_ret > 0)
-		return lame_encode_buffer (ices_reencode_flags, left, right, decode_ret, (char *)outbuf, outlen);
-	return decode_ret;
+  if (nsamples <= 0)
+    return nsamples;
+  
+  return lame_encode_buffer (lame, left, right, nsamples, outbuf, outlen);
 }
 
 /* At EOF of each file, flush the liblame buffers and get some extra candy */
 int
-ices_reencode_flush (unsigned char *outbuf, int maxlen)
+ices_reencode_flush (ices_stream_config_t* stream, unsigned char *outbuf,
+		     int maxlen)
 {
-	int ret = lame_encode_flush_nogap (ices_reencode_flags, (char *)outbuf, maxlen);
-	return ret;
+  lame_global_flags* lame = (lame_global_flags*) stream->encoder_state;
+  int rc;
+
+  rc = lame_encode_flush_nogap (lame, outbuf, maxlen);
+  stream->encoder_initialised = 0;
+
+  return rc;
 }
 #endif
Index: src/reencode.h
===================================================================
RCS file: /cvsroot/ices/src/reencode.h,v
retrieving revision 1.2
diff -u -p -r1.2 reencode.h
--- src/reencode.h	2000/08/29 17:37:24	1.2
+++ src/reencode.h	2001/06/10 16:39:24
@@ -21,10 +21,14 @@
 /* Public function declarations */
 void ices_reencode_initialize (void);
 void ices_reencode_shutdown (void);
-int ices_reencode_reencode_chunk (unsigned char *buff, int buflen, unsigned char *outbuf, int outlen);
+int ices_reencode_decode (unsigned char* buf, size_t blen);
+int ices_reencode_reencode_chunk (ices_stream_config_t* stream,
+				  unsigned char *buff, int buflen,
+				  unsigned char *outbuf, int outlen);
 void ices_reencode_set_channels (int channels);
 void ices_reencode_set_sample_rate (int samplerate);
 void ices_reencode_set_mode (int mode);
 void ices_reencode_reset (void);
-int ices_reencode_flush (unsigned char *outbuf, int maxlen);
+int ices_reencode_flush (ices_stream_config_t* stream, unsigned char *outbuf,
+			 int maxlen);
 
Index: src/setup.c
===================================================================
RCS file: /cvsroot/ices/src/setup.c,v
retrieving revision 1.21
diff -u -p -r1.21 setup.c
--- src/setup.c	2001/06/08 19:49:42	1.21
+++ src/setup.c	2001/06/10 16:39:24
@@ -28,7 +28,7 @@ static void ices_setup_parse_defaults (i
 static void ices_setup_parse_config_file (ices_config_t *ices_config, const char *configfile);
 static void ices_setup_parse_command_line (ices_config_t *ices_config, char **argv, int argc);
 static void ices_setup_parse_command_line_for_new_configfile (ices_config_t *ices_config, char **argv, int argc);
-static void ices_setup_activate_libshout_changes (shout_conn_t *conn, const ices_config_t *ices_config);
+static void ices_setup_activate_libshout_changes (const ices_config_t *ices_config);
 static void ices_setup_usage (void);
 static void ices_setup_update_pidfile (int icespid);
 static void ices_setup_daemonize (void);
@@ -36,7 +36,6 @@ static void ices_setup_run_mode_select (
 static int  ices_setup_shutting_down = 0;
 static void ices_setup_free_all_allocations (ices_config_t *ices_config);
 
-extern shout_conn_t conn;
 extern ices_config_t ices_config;
 
 static int ThreadsInitialised = 0;
@@ -49,42 +48,44 @@ static int ThreadsInitialised = 0;
 void
 ices_setup_initialize (void)
 {
-	/* Setup signal handlers */
-	ices_signals_setup ();
+  ices_stream_config_t* stream;
+  
+  /* Setup signal handlers */
+  ices_signals_setup ();
 
-	/* Parse the options in the config file, and the command line */
-	ices_setup_parse_options (&ices_config);
+  /* Parse the options in the config file, and the command line */
+  ices_setup_parse_options (&ices_config);
 
-	/* Go into daemon mode if requested */
-	ices_setup_run_mode_select (&ices_config);
+  /* Go into daemon mode if requested */
+  ices_setup_run_mode_select (&ices_config);
 
-	/* Initialize the libshout structure */
-	shout_init_connection (&conn);
-	
-	/* Copy those options to the libshout structure */
-	ices_setup_activate_libshout_changes (&conn, &ices_config);
+  /* Initialize the libshout structure */
+  for (stream = ices_config.streams; stream; stream = stream->next) {
+    shout_init_connection (&stream->conn);
+  }
+
+  ices_setup_activate_libshout_changes (&ices_config);
 
-	/* Open logfiles */
-	ices_log_initialize ();
+  /* Open logfiles */
+  ices_log_initialize ();
 
-	/* Initialize the interpreters */
-	interpreter_initialize ();
+  /* Initialize the interpreters */
+  interpreter_initialize ();
 
-	/* Initialize the thread library */
-	thread_initialize ();
-	ThreadsInitialised = 1;
+  /* Initialize the thread library */
+  thread_initialize ();
+  ThreadsInitialised = 1;
 
-	/* Initialize the playlist handler */
-	ices_playlist_initialize ();
+  /* Initialize the playlist handler */
+  ices_playlist_initialize ();
 
-	/* Initialize id3 stuff */
-	ices_id3_initialize ();
+  /* Initialize id3 stuff */
+  ices_id3_initialize ();
 
 #ifdef HAVE_LIBLAME
-	/* Initialize liblame for reeencoding */
-	ices_reencode_initialize ();
+  /* Initialize liblame for reeencoding */
+  ices_reencode_initialize ();
 #endif
-	
 }
 
 /* Top level ices shutdown function.
@@ -92,55 +93,58 @@ ices_setup_initialize (void)
 void
 ices_setup_shutdown (void)
 {
-	/* Protection for multiple threads calling shutdown.
-	 * Remember that this is can be called from many places,
-	 * including the SIGING signal handler */
-	if (ThreadsInitialised) {
-		thread_library_lock ();
-
-		if (ices_setup_shutting_down) {
-		  thread_library_unlock ();
-		  return;
-		}
-
-		ices_setup_shutting_down = 1;
-
-		thread_library_unlock ();
-	}
+  ices_stream_config_t* stream;
 
-	/* Tell libshout to disconnect from server */
-	shout_disconnect (&conn);
+  /* Protection for multiple threads calling shutdown.
+   * Remember that this is can be called from many places,
+   * including the SIGING signal handler */
+  if (ThreadsInitialised) {
+    thread_library_lock ();
+
+    if (ices_setup_shutting_down) {
+      thread_library_unlock ();
+      return;
+    }
+
+    ices_setup_shutting_down = 1;
+
+    thread_library_unlock ();
+  }
+
+  /* Tell libshout to disconnect from server */
+  for (stream = ices_config.streams; stream; stream = stream->next)
+    shout_disconnect (&stream->conn);
 
 #ifdef HAVE_LIBLAME
-	/* Order the reencoding engine to shutdown */
-	ices_reencode_shutdown ();
+  /* Order the reencoding engine to shutdown */
+  ices_reencode_shutdown ();
 #endif
 
-	/* Tell the playlist module to shutdown and cleanup */
-	ices_playlist_shutdown ();
+  /* Tell the playlist module to shutdown and cleanup */
+  ices_playlist_shutdown ();
 
-	/* Shutdown id3 module */
-	ices_id3_shutdown ();
+  /* Shutdown id3 module */
+  ices_id3_shutdown ();
 
-	/* Cleanup the cue file (the cue module has no init yet) */
-	ices_cue_shutdown ();
+  /* Cleanup the cue file (the cue module has no init yet) */
+  ices_cue_shutdown ();
 
-	/* Shutdown the thread library */
-	thread_shutdown ();
-
-	/* Make sure we're not leaving any memory allocated around when
-	 * we exit. This makes it easier to find memory leaks, and 
-	 * some systems actually don't clean up that well */
-	ices_setup_free_all_allocations (&ices_config);
+  /* Shutdown the thread library */
+  thread_shutdown ();
+
+  /* Make sure we're not leaving any memory allocated around when
+   * we exit. This makes it easier to find memory leaks, and 
+   * some systems actually don't clean up that well */
+  ices_setup_free_all_allocations (&ices_config);
 	
-	/* Let the log and console know we wen't down ok */
-	ices_log ("Ices Exiting...");
+  /* Let the log and console know we wen't down ok */
+  ices_log ("Ices Exiting...");
 
-	/* Close logfiles */
-	ices_log_shutdown ();
+  /* Close logfiles */
+  ices_log_shutdown ();
 	
-	/* Down and down we go... */
-	exit (1);
+  /* Down and down we go... */
+  exit (1);
 }
 
 /* Local function definitions */
@@ -171,72 +175,85 @@ ices_setup_parse_options (ices_config_t 
 static void
 ices_setup_parse_defaults (ices_config_t *ices_config)
 {
-	ices_config->host = ices_util_strdup (ICES_DEFAULT_HOST);
-	ices_config->port = ICES_DEFAULT_PORT;
-	ices_config->mount = ices_util_strdup (ICES_DEFAULT_MOUNT);
-	ices_config->password = ices_util_strdup (ICES_DEFAULT_PASSWORD);
-	ices_config->header_protocol = ICES_DEFAULT_HEADER_PROTOCOL;
-	ices_config->name = ices_util_strdup (ICES_DEFAULT_NAME);
-	ices_config->genre = ices_util_strdup (ICES_DEFAULT_GENRE);
-	ices_config->description = ices_util_strdup (ICES_DEFAULT_DESCRIPTION);
-	ices_config->url = ices_util_strdup (ICES_DEFAULT_URL);
-	ices_config->bitrate = ICES_DEFAULT_BITRATE;
-	ices_config->ispublic = ICES_DEFAULT_ISPUBLIC;
-	ices_config->dumpfile = NULL; /* No dumpfile by default */
-	ices_config->configfile = ices_util_strdup (ICES_DEFAULT_CONFIGFILE);
-	ices_config->playlist_file = ices_util_strdup (ICES_DEFAULT_PLAYLIST_FILE);
-	ices_config->interpreter_file = NULL; /* Default to the hardcoded default */
-	ices_config->randomize_playlist = ICES_DEFAULT_RANDOMIZE_PLAYLIST;
-	ices_config->daemon = ICES_DEFAULT_DAEMON;
-	ices_config->pre_dj = ICES_DEFAULT_PRE_DJ;
-	ices_config->post_dj = ICES_DEFAULT_POST_DJ;
-	ices_config->base_directory = ices_util_strdup (ICES_DEFAULT_BASE_DIRECTORY);
-	ices_config->playlist_type = ICES_DEFAULT_PLAYLIST_TYPE;
-	ices_config->verbose = ICES_DEFAULT_VERBOSE;
-	ices_config->reencode = ICES_DEFAULT_REENCODE;
-	ices_config->out_numchannels = -1;
-	ices_config->out_samplerate = -1;
-}
-
-/* Function to free() all allocated memory when ices shuts down. */
-static void
-ices_setup_free_all_allocations (ices_config_t *ices_config)
-{
-	if (ices_config->host)
-		ices_util_free (ices_config->host);
-
-	if (ices_config->mount)
-		ices_util_free (ices_config->mount);
+  ices_config->host = ices_util_strdup (ICES_DEFAULT_HOST);
+  ices_config->port = ICES_DEFAULT_PORT;
+  ices_config->password = ices_util_strdup (ICES_DEFAULT_PASSWORD);
+  ices_config->header_protocol = ICES_DEFAULT_HEADER_PROTOCOL;
+  ices_config->dumpfile = NULL; /* No dumpfile by default */
+  ices_config->configfile = ices_util_strdup (ICES_DEFAULT_CONFIGFILE);
+  ices_config->playlist_file = ices_util_strdup (ICES_DEFAULT_PLAYLIST_FILE);
+  ices_config->interpreter_file = NULL; /* Default to the hardcoded default */
+  ices_config->randomize_playlist = ICES_DEFAULT_RANDOMIZE_PLAYLIST;
+  ices_config->daemon = ICES_DEFAULT_DAEMON;
+  ices_config->pre_dj = ICES_DEFAULT_PRE_DJ;
+  ices_config->post_dj = ICES_DEFAULT_POST_DJ;
+  ices_config->base_directory = ices_util_strdup (ICES_DEFAULT_BASE_DIRECTORY);
+  ices_config->playlist_type = ICES_DEFAULT_PLAYLIST_TYPE;
+  ices_config->verbose = ICES_DEFAULT_VERBOSE;
+  ices_config->reencode = 0;
 
-	if (ices_config->password)
-		ices_util_free (ices_config->password);
+  ices_config->streams =
+    (ices_stream_config_t*) malloc (sizeof (ices_stream_config_t));
 
-	if (ices_config->name)
-		ices_util_free (ices_config->name);
-
-	if (ices_config->genre)
-		ices_util_free (ices_config->genre);
+  ices_setup_parse_stream_defaults (ices_config->streams);
+}
 
-	if (ices_config->description)
-		ices_util_free (ices_config->description);
+/* Place hardcoded defaults into an ices_stream_config object */
+void
+ices_setup_parse_stream_defaults (ices_stream_config_t* ices_stream_config)
+{
+  ices_stream_config->mount = ices_util_strdup (ICES_DEFAULT_MOUNT);
+  
+  ices_stream_config->name = ices_util_strdup (ICES_DEFAULT_NAME);
+  ices_stream_config->genre = ices_util_strdup (ICES_DEFAULT_GENRE);
+  ices_stream_config->description = ices_util_strdup (ICES_DEFAULT_DESCRIPTION);
+  ices_stream_config->url = ices_util_strdup (ICES_DEFAULT_URL);
+  ices_stream_config->ispublic = ICES_DEFAULT_ISPUBLIC;
+
+  ices_stream_config->bitrate = ICES_DEFAULT_BITRATE;
+  ices_stream_config->reencode = ICES_DEFAULT_REENCODE;
+  ices_stream_config->out_numchannels = -1;
+  ices_stream_config->out_samplerate = -1;
 
-	if (ices_config->url)
-		ices_util_free (ices_config->url);
+  ices_stream_config->encoder_state = NULL;
+  ices_stream_config->encoder_initialised = 0;
 
-	if (ices_config->dumpfile)
-		ices_util_free (ices_config->dumpfile);
+  ices_stream_config->next = NULL;
+}
 
-	if (ices_config->configfile)
-		ices_util_free (ices_config->configfile);
+/* Frees ices_stream_config_t data (but not the object itself) */
+static void
+ices_setup_free_stream (ices_stream_config_t* stream)
+{
+  ices_util_free (stream->mount);
 
-	if (ices_config->playlist_file)
-		ices_util_free (ices_config->playlist_file);
+  ices_util_free (stream->name);
+  ices_util_free (stream->genre);
+  ices_util_free (stream->description);
+  ices_util_free (stream->url);
+}
 
-	if (ices_config->interpreter_file)
-		ices_util_free (ices_config->interpreter_file);
+/* Function to free() all allocated memory when ices shuts down. */
+static void
+ices_setup_free_all_allocations (ices_config_t *ices_config)
+{
+  ices_stream_config_t *stream, *next;
 
-	if (ices_config->base_directory)
-		ices_util_free (ices_config->base_directory);
+  ices_util_free (ices_config->host);
+  ices_util_free (ices_config->password);
+  ices_util_free (ices_config->dumpfile);
+  ices_util_free (ices_config->configfile);
+  ices_util_free (ices_config->playlist_file);
+  ices_util_free (ices_config->interpreter_file);
+  ices_util_free (ices_config->base_directory);
+
+  for (stream = ices_config->streams; stream; stream = next)
+  {
+    next = stream->next;
+
+    ices_setup_free_stream (stream);
+    ices_util_free (stream);
+  }
 }
 
 /* Tell the xml module to parse the config file. */
@@ -328,16 +345,15 @@ ices_setup_parse_command_line (ices_conf
 					break;
 				case 'b':
 					arg++;
-					ices_config->bitrate = atoi (argv[arg]);
+					ices_config->streams->bitrate = atoi (argv[arg]);
 					break;
 				case 'c':
 					arg++;
 					break;
 				case 'd':
 					arg++;
-					if (ices_config->description)
-						ices_util_free (ices_config->description);
-					ices_config->description = ices_util_strdup (argv[arg]);
+					ices_util_free (ices_config->streams->description);
+					ices_config->streams->description = ices_util_strdup (argv[arg]);
 					break;
 				case 'D':
 					arg++;
@@ -359,9 +375,8 @@ ices_setup_parse_command_line (ices_conf
 					break;
 				case 'g':
 					arg++;
-					if (ices_config->genre)
-						ices_util_free (ices_config->genre);
-					ices_config->genre = ices_util_strdup (argv[arg]);
+					ices_util_free (ices_config->streams->genre);
+					ices_config->streams->genre = ices_util_strdup (argv[arg]);
 					break;
 				case 'h':
 					arg++;
@@ -371,7 +386,7 @@ ices_setup_parse_command_line (ices_conf
 					break;
 				case 'H':
 					arg++;
-					ices_config->out_samplerate = atoi (argv[arg]);
+					ices_config->streams->out_samplerate = atoi (argv[arg]);
 					break;
 				case 'i':
 					ices_config->header_protocol = icy_header_protocol_e;
@@ -384,19 +399,17 @@ ices_setup_parse_command_line (ices_conf
 					break;
 				case 'm':
 					arg++;
-					if (ices_config->mount)
-						ices_util_free (ices_config->mount);
-					ices_config->mount = ices_util_strdup (argv[arg]);
+					ices_util_free (ices_config->streams->mount);
+					ices_config->streams->mount = ices_util_strdup (argv[arg]);
 					break;
 				case 'N':
 					arg++;
-					ices_config->out_numchannels = atoi (argv[arg]);
+					ices_config->streams->out_numchannels = atoi (argv[arg]);
 					break;
 				case 'n':
 					arg++;
-					if (ices_config->name)
-						ices_util_free (ices_config->name);
-					ices_config->name = ices_util_strdup (argv[arg]);
+					ices_util_free (ices_config->streams->name);
+					ices_config->streams->name = ices_util_strdup (argv[arg]);
 					break;
 				case 'P':
 					arg++;
@@ -410,7 +423,7 @@ ices_setup_parse_command_line (ices_conf
 					break;
 				case 'R':
 #ifdef HAVE_LIBLAME
-					ices_config->reencode = 1;
+					ices_config->streams->reencode = 1;
 #else
 					ices_log ("Support for reencoding with liblame was not found. You can't reencode this.");
 					ices_setup_shutdown ();
@@ -431,13 +444,12 @@ ices_setup_parse_command_line (ices_conf
 					break;
 				case 's':
 					arg++;
-					ices_config->ispublic = 0;
+					ices_config->streams->ispublic = 0;
 					break;
 				case 'u':
 					arg++;
-					if (ices_config->url)
-						ices_util_free (ices_config->url);
-					ices_config->url = ices_util_strdup (argv[arg]);
+					ices_util_free (ices_config->streams->url);
+					ices_config->streams->url = ices_util_strdup (argv[arg]);
 					break;
 				case 'v':
 					ices_config->verbose = 1;
@@ -460,28 +472,42 @@ ices_setup_parse_command_line (ices_conf
 /* This function takes all the new configuration and copies it to the
    libshout object. */
 static void
-ices_setup_activate_libshout_changes (shout_conn_t *conn, const ices_config_t *ices_config)
+ices_setup_activate_libshout_changes (const ices_config_t *ices_config)
 {
-	conn->port = ices_config->port;
-	conn->ip = ices_config->host;
-	conn->password = ices_config->password;
-	conn->icy_compat = ices_config->header_protocol == icy_header_protocol_e;
-	conn->dumpfile = ices_config->dumpfile;
-	conn->name = ices_config->name;
-	conn->url = ices_config->url;
-	conn->genre = ices_config->genre;
-	conn->description = ices_config->description;
-	conn->bitrate = ices_config->bitrate;
-	conn->ispublic = ices_config->ispublic;
-	conn->mount = ices_config->mount;
-
-	ices_log_debug ("Sending following information to libshout:");
-	ices_log_debug ("Host: %s\tPort: %d", conn->ip, conn->port);
-	ices_log_debug ("Password: %s\tIcy Compat: %d", conn->password, conn->icy_compat);
-	ices_log_debug ("Name: %s\tURL: %s", conn->name, conn->url);
-	ices_log_debug ("Genre: %s\tDesc: %s", conn->genre, conn->description);
-	ices_log_debug ("Bitrate: %d\tPublic: %d", conn->bitrate, conn->ispublic);
-	ices_log_debug ("Mount: %s\tDumpfile: %s", ices_util_nullcheck (conn->mount), ices_util_nullcheck (conn->dumpfile)); 
+  ices_stream_config_t* stream;
+  int streamno = 0;
+
+  for (stream = ices_config->streams; stream; stream = stream->next) {
+    stream->conn.port = ices_config->port;
+    stream->conn.ip = ices_config->host;
+    stream->conn.password = ices_config->password;
+    stream->conn.icy_compat =
+      ices_config->header_protocol == icy_header_protocol_e;
+    stream->conn.dumpfile = ices_config->dumpfile;
+    stream->conn.name = stream->name;
+    stream->conn.url = stream->url;
+    stream->conn.genre = stream->genre;
+    stream->conn.description = stream->description;
+    stream->conn.bitrate = stream->bitrate;
+    stream->conn.ispublic = stream->ispublic;
+    stream->conn.mount = stream->mount;
+
+    ices_log_debug ("Sending following information to libshout:");
+    ices_log_debug ("Stream: %d", streamno);
+    ices_log_debug ("Host: %s\tPort: %d", stream->conn.ip, stream->conn.port);
+    ices_log_debug ("Password: %s\tIcy Compat: %d", stream->conn.password,
+		    stream->conn.icy_compat);
+    ices_log_debug ("Name: %s\tURL: %s", stream->conn.name, stream->conn.url);
+    ices_log_debug ("Genre: %s\tDesc: %s", stream->conn.genre,
+		    stream->conn.description);
+    ices_log_debug ("Bitrate: %d\tPublic: %d", stream->conn.bitrate,
+		    stream->conn.ispublic);
+    ices_log_debug ("Mount: %s\tDumpfile: %s",
+		    ices_util_nullcheck (stream->conn.mount),
+		    ices_util_nullcheck (stream->conn.dumpfile));
+
+    streamno++;
+  }
 }
 
 /* Display all command line options for ices */
Index: src/setup.h
===================================================================
RCS file: /cvsroot/ices/src/setup.h,v
retrieving revision 1.3
diff -u -p -r1.3 setup.h
--- src/setup.h	2000/08/29 17:37:24	1.3
+++ src/setup.h	2001/06/10 16:39:24
@@ -21,3 +21,6 @@
 /* Public function declarations */
 void ices_setup_initialize (void);
 void ices_setup_shutdown (void);
+
+/* exported for the config parser */
+void ices_setup_parse_stream_defaults (ices_stream_config_t*);
Index: src/stream.c
===================================================================
RCS file: /cvsroot/ices/src/stream.c,v
retrieving revision 1.15
diff -u -p -r1.15 stream.c
--- src/stream.c	2000/08/29 17:37:24	1.15
+++ src/stream.c	2001/06/10 16:39:24
@@ -22,14 +22,15 @@
 
 #include <resolver.h>
 
-extern shout_conn_t conn;
 extern ices_config_t ices_config;
 
 /* Private function declarations */
 static int ices_stream_send_file (const char *file);
 static int ices_stream_fd_size (int fd, const char *file, int file_bytes);
 static int ices_stream_fd_until_eof (int fd, const char *file);
-static int ices_stream_send_reencoded (shout_conn_t *conn, unsigned char *buff, int read_bytes, int file_bytes);
+static int ices_stream_send_reencoded (ices_stream_config_t* stream,
+				       unsigned char *buff, int read_bytes,
+				       int file_bytes);
 
 /* Public function definitions */
 
@@ -38,94 +39,102 @@ static int ices_stream_send_reencoded (s
 void
 ices_stream_loop (void)
 {
-	char namespace[1024];
-	int consecutive_errors = 0;
-
-	/* Check if user gave us a hostname */
-	if (!isdigit ((int)(conn.ip[strlen (conn.ip) - 1]))) {
-		char hostbuff[1024];
-
-		/* Ok, he did, let's get the ip */
-		conn.ip = resolver_getip (conn.ip, hostbuff, 1024);
-		if (conn.ip == NULL) {
-			ices_log ("Failed resolving servername.");
-			ices_setup_shutdown ();
-		}
-	}
-
-	/* Tell libshout to connect to the server */
-	if (!shout_connect (&conn)) {
-		ices_log ("Failed connecting to server %s, error: %s", conn.ip, shout_strerror(&conn, conn.error, namespace, 1024));
-		ices_setup_shutdown ();
-	}
+  char namespace[1024];
+  int consecutive_errors = 0;
+  ices_stream_config_t* stream;
+
+  /* Check if user gave us a hostname */
+  for (stream = ices_config.streams; stream; stream = stream->next) {
+    if (!isdigit ((int)(stream->conn.ip[strlen (stream->conn.ip) - 1]))) {
+      char hostbuff[1024];
+
+      /* Ok, he did, let's get the ip */
+      stream->conn.ip = resolver_getip (stream->conn.ip, hostbuff, 1024);
+      if (stream->conn.ip == NULL) {
+	ices_log ("Failed resolving servername.");
+	ices_setup_shutdown ();
+      }
+    }
+  }
+
+  /* Tell libshout to connect to the server */
+  for (stream = ices_config.streams; stream; stream = stream->next) {
+    if (!shout_connect (&stream->conn)) {
+      ices_log ("Failed connecting to server %s, error: %s", stream->conn.ip, shout_strerror(&stream->conn, stream->conn.error, namespace, 1024));
+      ices_setup_shutdown ();
+    }
+  }
 	
-	ices_log ("Connected to server %s...", conn.ip);
+  ices_log ("Connected to server %s...", ices_config.streams->conn.ip);
 
-	/* Foreeeever streams, I want to be forever streaming... */
-	while (1) {
-		char *file;
-
-		/* NULL definition as of now, it will handle scripted
-		 * DJ files in the future. Or perhaps there is no need
-		 * for this now that all playlist handling is done in
-		 * scripts. */
-		ices_dj_pre ();
-
-		/* Get the next file to stream from the playlist module.
-		 * This is a dynamically allocated string, which we free
-		 * further down in the loop */
-		file = ices_playlist_get_next ();
+  /* Foreeeever streams, I want to be forever streaming... */
+  while (1) {
+    char *file;
+
+    /* NULL definition as of now, it will handle scripted
+     * DJ files in the future. Or perhaps there is no need
+     * for this now that all playlist handling is done in
+     * scripts. */
+    ices_dj_pre ();
+
+    /* Get the next file to stream from the playlist module.
+     * This is a dynamically allocated string, which we free
+     * further down in the loop */
+    file = ices_playlist_get_next ();
 		
-		/* We quit if the playlist handler gives us a NULL filename */
-		if (!file) {
-			ices_log ("Warning: ices_file_get_next() gave me an error, this is not good. [%s]", ices_log_get_error ());
-			ices_setup_shutdown ();
-		}
+    /* We quit if the playlist handler gives us a NULL filename */
+    if (!file) {
+      ices_log ("Warning: ices_file_get_next() gave me an error, this is not good. [%s]", ices_log_get_error ());
+      ices_setup_shutdown ();
+    }
 		
-		/* If something goes on while transfering, we just go on */
-		if (!ices_stream_send_file (file)) {
-			ices_log ("Warning: Encountered error while transfering %s. [%s]", file, ices_log_get_error ());
-
-			consecutive_errors++;
-
-			/* Let's see if it was a libshout socket error */
-			if (conn.error == SHOUTERR_SOCKET) {
-				ices_log ("Libshout communication problem, trying to reconnect to server");
-				
-				/* Make sure we disconnect */
-				shout_disconnect (&conn);
+    /* If something goes on while transfering, we just go on */
+    if (!ices_stream_send_file (file)) {
+      ices_log ("Warning: Encountered error while transfering %s. [%s]", file, ices_log_get_error ());
+
+      consecutive_errors++;
+
+      /* Not the best interface - we have to search for the error */
+      /* Let's see if it was a libshout socket error */
+      for (stream = ices_config.streams; stream; stream = stream->next) {
+	if (stream->conn.error == SHOUTERR_SOCKET) {
+	  ices_log ("Libshout communication problem, trying to reconnect to server");
 				
-				/* Reconnect */
-				if (!shout_connect (&conn)) {
-					ices_log ("Failed reconnecting to server %s, error: %i", conn.ip, conn.error);
-					ices_setup_shutdown ();
-				}
-			}
+	  /* Make sure we disconnect */
+	  shout_disconnect (&stream->conn);
+
+	  /* Reconnect */
+	  if (!shout_connect (&stream->conn)) {
+	    ices_log ("Failed reconnecting to server %s, error: %i", stream->conn.ip, stream->conn.error);
+	    ices_setup_shutdown ();
+	  }
+	}
+      }
 			
-			/* This stops ices from entering a loop with 10-20 lines of output per
-			   second. Usually caused by a playlist handler that produces only
-			   invalid file names. */
-			if (consecutive_errors > 10) {
-				ices_log ("Exiting after 10 consecutive errors.");
-				ices_util_free (file);
-				ices_setup_shutdown ();
-			}
-
-			ices_util_free (file);
-			continue;
-		} else {
-			/* Reset the consecutive error counter */
-			consecutive_errors = 0;
-		}
+      /* This stops ices from entering a loop with 10-20 lines of output per
+	 second. Usually caused by a playlist handler that produces only
+	 invalid file names. */
+      if (consecutive_errors > 10) {
+	ices_log ("Exiting after 10 consecutive errors.");
+	ices_util_free (file);
+	ices_setup_shutdown ();
+      }
+
+      ices_util_free (file);
+      continue;
+    } else {
+      /* Reset the consecutive error counter */
+      consecutive_errors = 0;
+    }
 		
-		/* Run the post DJ program/script
-		 * Currently does nada */
-		ices_dj_post ();
+    /* Run the post DJ program/script
+     * Currently does nada */
+    ices_dj_post ();
 		
-		/* Free the dynamically allocated filename */
-		ices_util_free (file);
-	}
-	/* Not reached */
+    /* Free the dynamically allocated filename */
+    ices_util_free (file);
+  }
+  /* Not reached */
 }
 
 /* This function is called to stream a single file */
@@ -206,117 +215,130 @@ ices_stream_send_file (const char *file)
 static int
 ices_stream_fd_size (int fd, const char *file, int file_bytes)
 {
-	int read_bytes, ret = -1, orig_size = file_bytes;
-	int bitrate = ices_mp3_get_bitrate ();
-	unsigned char buff[4096];
-	char namespace[1024];
+  ices_stream_config_t* stream;
+  int read_bytes, ret = -1, orig_size = file_bytes;
+  int bitrate = ices_mp3_get_bitrate ();
+  unsigned char buff[4096];
+  char namespace[1024];
 
-	ices_log ("Streaming %d bytes from file %s", file_bytes, file);
+  ices_log ("Streaming %d bytes from file %s", file_bytes, file);
 
 #ifdef HAVE_LIBLAME
-	if (ices_config.reencode) {
-		ices_reencode_reset ();
-	}
+  ices_reencode_reset ();
 #endif
 
-	while (file_bytes >= 0) {
-		read_bytes = read (fd, buff, 4096);
+  while (file_bytes >= 0) {
+    read_bytes = read (fd, buff, 4096);
 		
-		if (read_bytes > 0) {
+    if (read_bytes > 0) {
 
-			file_bytes -= read_bytes;
+      file_bytes -= read_bytes;
 
-			if (ices_config.reencode) {
-				/* Users wants us to reencode to different
-				 * bitrate, this reencodes and sends the
-				 * reencoded buff to the server */
-				ret = ices_stream_send_reencoded (&conn, buff, read_bytes, file_bytes);
-			} else {
-				/* Send read data as-is to the server */
-				ret = shout_send_data (&conn, buff, read_bytes);
-			}
-
-			if (!ret) {
-				ices_log_error ("Libshout reported send error: %s", shout_strerror (&conn, conn.error, namespace, 1024));
-				return 0;
-			}
-			
-		} else if (read_bytes == 0) 
-			return 1;
-		else {
-			ices_log_error ("Read error while reading %s: %s", file, ices_util_strerror (errno, namespace, 1024));
-			return 0;
-		}
-
-		/* Update cue file */
-		ices_cue_update (file, orig_size, bitrate, orig_size - file_bytes);
-		/* Give the libshout module a chance to sleep */
-		shout_sleep(&conn);
-	}
+#ifdef HAVE_LIBLAME
+      if (ices_config.reencode)
+	ices_reencode_decode (buff, read_bytes);
+#endif
+      for (stream = ices_config.streams; stream; stream = stream->next) {
+	if (stream->reencode) {
+	  /* Users wants us to reencode to different
+	   * bitrate, this reencodes and sends the
+	   * reencoded buff to the server */
+	  ret = ices_stream_send_reencoded(stream, buff, read_bytes, file_bytes);
+	} else {
+	  /* Send read data as-is to the server */
+	  ret = shout_send_data (&stream->conn, buff, read_bytes);
+	}
+
+	if (!ret) {
+	  ices_log_error ("Libshout reported send error: %s", shout_strerror (&stream->conn, stream->conn.error, namespace, 1024));
+	  return 0;
+	}
+      }
+      
+    } else if (read_bytes == 0) 
+      return 1;
+    else {
+      ices_log_error ("Read error while reading %s: %s", file, ices_util_strerror (errno, namespace, 1024));
+      return 0;
+    }
+
+    /* Update cue file */
+    ices_cue_update (file, orig_size, bitrate, orig_size - file_bytes);
+    /* Give the libshout module a chance to sleep */
+    for (stream = ices_config.streams; stream; stream = stream->next)
+      shout_sleep(&stream->conn);
+  }
 
-	/* All is fine */
-	return 1;
+  /* All is fine */
+  return 1;
 }
 
 /* This function is used when streaming from fifo or pipe */
 static int
 ices_stream_fd_until_eof (int fd, const char *file)
 {
-	int file_bytes_read, read_bytes, ret;
-        unsigned char buff[4096];
-	char namespace[1024];
+  ices_stream_config_t* stream;
+  int file_bytes_read, read_bytes, ret;
+  unsigned char buff[4096];
+  char namespace[1024];
 
-	ices_log ("Streaming from %s until EOF..", file);
+  ices_log ("Streaming from %s until EOF..", file);
 	
-        file_bytes_read = 0;
+  file_bytes_read = 0;
 	
-        while (1) {
-                read_bytes = read (fd, buff, 4096);
+  while (1) {
+    read_bytes = read (fd, buff, 4096);
 		
-                if (read_bytes > 0) {
+    if (read_bytes > 0) {
 			
-                        file_bytes_read = file_bytes_read + read_bytes;
-                        ret = shout_send_data (&conn, buff, read_bytes);
+      file_bytes_read = file_bytes_read + read_bytes;
+
+      for (stream = ices_config.streams; stream; stream = stream->next) {
+	ret = shout_send_data (&stream->conn, buff, read_bytes);
 			
-                        if (!ret) {
-                                ices_log_error ("Libshout reported send error: %s", shout_strerror (&conn, conn.error, namespace, 1024));
-                                break;
-                        }
+	if (!ret) {
+	  ices_log_error ("Libshout reported send error: %s", shout_strerror (&stream->conn, stream->conn.error, namespace, 1024));
+	  break;
+	}
+      }
 			
-                } else if (read_bytes == 0)
-                        return 1;
-                else {
-                        ices_log_error ("Read error while reading %s: %s", file, ices_util_strerror (errno, namespace, 1024));
-                        return 0;
-                }
-                shout_sleep(&conn);
-        }
-	return 1;
+    } else if (read_bytes == 0)
+      return 1;
+    else {
+      ices_log_error ("Read error while reading %s: %s", file, ices_util_strerror (errno, namespace, 1024));
+      return 0;
+    }
+
+    for (stream = ices_config.streams; stream; stream = stream->next)
+      shout_sleep(&stream->conn);
+  }
+  return 1;
 }
 
 /* Reencode and send given chunk to server.
  * If file is up, flush the reencoding buffers and send the remains */
 static int
-ices_stream_send_reencoded (shout_conn_t *conn, unsigned char *buff, int read_bytes, int file_bytes)
+ices_stream_send_reencoded (ices_stream_config_t* stream, unsigned char *buff,
+			    int read_bytes, int file_bytes)
 {
-	int ret = 1;
+  int ret = 1;
 #ifdef HAVE_LIBLAME
-	unsigned char reencode_buff[15000];
-	int len = ices_reencode_reencode_chunk (buff, read_bytes, reencode_buff, 15000);
+  unsigned char reencode_buff[15000];
+  int len = ices_reencode_reencode_chunk (stream, buff, read_bytes, reencode_buff, 15000);
 	
-	if (len > 0)
-		ret = shout_send_data (conn, reencode_buff, len);
-	else if (len < 0)
-		return 0;
+  if (len > 0)
+    ret = shout_send_data (&stream->conn, reencode_buff, len);
+  else if (len < 0)
+    return 0;
 	
-	/* Flush and send remains if file is up */
-	if (file_bytes <= 0) {
-		len = ices_reencode_flush (reencode_buff, 15000);
-		if (len > 0)
-			ret = shout_send_data (conn, reencode_buff, len);
-	}
+  /* Flush and send remains if file is up */
+  if (file_bytes <= 0) {
+    len = ices_reencode_flush (stream, reencode_buff, 15000);
+    if (len > 0)
+      ret = shout_send_data (&stream->conn, reencode_buff, len);
+  }
 #endif
-	return ret;
+  return ret;
 }
 
 
Index: src/util.c
===================================================================
RCS file: /cvsroot/ices/src/util.c,v
retrieving revision 1.8
diff -u -p -r1.8 util.c
--- src/util.c	2001/01/09 02:13:32	1.8
+++ src/util.c	2001/06/10 16:39:25
@@ -23,7 +23,6 @@
 #include <string.h>
 #include <thread.h>
 
-extern shout_conn_t conn;
 extern ices_config_t ices_config;
 
 char **ices_argv;
Index: src/xml/parse.c
===================================================================
RCS file: /cvsroot/ices/src/xml/parse.c,v
retrieving revision 1.13
diff -u -p -r1.13 parse.c
--- src/xml/parse.c	2000/08/30 16:28:47	1.13
+++ src/xml/parse.c	2001/06/10 16:39:25
@@ -40,7 +40,7 @@ static void ices_xml_parse_setup_default
 static void ices_xml_parse_playlist_nodes (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur, ices_config_t *ices_config);
 static void ices_xml_parse_execution_nodes (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur, ices_config_t *ices_config);
 static void ices_xml_parse_server_nodes (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur, ices_config_t *ices_config);
-static void ices_xml_parse_stream_nodes (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur, ices_config_t *ices_config);
+static void ices_xml_parse_stream_nodes (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur, ices_stream_config_t *ices_stream_config);
 static xmlNodePtr ices_xml_get_root_element (xmlDocPtr doc);
 static xmlNodePtr ices_xml_get_children_node (xmlNodePtr cur);
 #endif
@@ -85,6 +85,10 @@ ices_xml_parse_file (const char *configf
 	xmlNsPtr ns;
 	xmlNodePtr cur;
 
+	/* we support multiple stream nodes */
+	int nstreams = 0;
+	ices_stream_config_t* stream = ices_config->streams;
+
 	/* This does the actual parsing */
 	if (!(doc = xmlParseFile (configfile))) {
 		ices_log_error ("XML Parser: Error while parsing %s", configfile);
@@ -132,21 +136,29 @@ ices_xml_parse_file (const char *configf
 	/* Separate the parsing into different submodules,
 	 * Server, Stream, Playlist and Execution */
 	while (cur != NULL) {
-
-		if (strcmp (cur->name, "Server") == 0) {
-			ices_xml_parse_server_nodes (doc, ns, ices_xml_get_children_node (cur), ices_config);
-		} else if (strcmp (cur->name, "Stream") == 0) {
-			ices_xml_parse_stream_nodes (doc, ns, ices_xml_get_children_node (cur), ices_config);
-		} else if (strcmp (cur->name, "Playlist") == 0) {
-			ices_xml_parse_playlist_nodes (doc, ns, ices_xml_get_children_node (cur), ices_config);
-		} else if (strcmp (cur->name, "Execution") == 0) {
-			ices_xml_parse_execution_nodes (doc, ns, ices_xml_get_children_node (cur), ices_config);
-		} else {
-			ices_log ("Unknown Node: %s", cur->name);
-		}
-
-		cur = cur->next;
+	  if (strcmp (cur->name, "Server") == 0) {
+	    ices_xml_parse_server_nodes (doc, ns, ices_xml_get_children_node (cur), ices_config);
+	  } else if (strcmp (cur->name, "Stream") == 0) {
+	    /* first stream is preallocated */
+	    if (nstreams) {
+	      stream->next = (ices_stream_config_t*) malloc (sizeof (ices_stream_config_t));
+	      stream = stream->next;
+	      /* in case fields are omitted in the config file */
+	      ices_setup_parse_stream_defaults (stream);
+	    }
+
+	    ices_xml_parse_stream_nodes (doc, ns, ices_xml_get_children_node (cur), stream);
+
+	    nstreams++;
+	  } else if (strcmp (cur->name, "Playlist") == 0) {
+	    ices_xml_parse_playlist_nodes (doc, ns, ices_xml_get_children_node (cur), ices_config);
+	  } else if (strcmp (cur->name, "Execution") == 0) {
+	    ices_xml_parse_execution_nodes (doc, ns, ices_xml_get_children_node (cur), ices_config);
+	  } else {
+	    ices_log ("Unknown Node: %s", cur->name);
+	  }
 
+	  cur = cur->next;
 	}
 	
 	/* Be a good boy and cleanup */
@@ -155,52 +167,51 @@ ices_xml_parse_file (const char *configf
 }
 
 /* Parse the stream specific configuration */
-static void
-ices_xml_parse_stream_nodes (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur, ices_config_t *ices_config)
+void
+ices_xml_parse_stream_nodes (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur,
+			     ices_stream_config_t *stream)
 {
-	while (cur != NULL) {
-		if (strcmp (cur->name, "Name") == 0) {
-
-			if (ices_config->name)
-				ices_util_free (ices_config->name);
-
-			ices_config->name = ices_util_strdup ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
-
-		} else if (strcmp (cur->name, "Genre") == 0) {
-
-			if (ices_config->genre)
-				ices_util_free (ices_config->genre);
-
-			ices_config->genre = ices_util_strdup ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
-
-		} else if (strcmp (cur->name, "Description") == 0) {
-
-			if (ices_config->description)
-				ices_util_free (ices_config->description);
-
-			ices_config->description = ices_util_strdup ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
-
-		} else if (strcmp (cur->name, "URL") == 0) {
-
-			if (ices_config->url)
-				ices_util_free (ices_config->url);
-
-			ices_config->url = ices_util_strdup ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
+  while (cur != NULL) {
+    if (strcmp (cur->name, "Name") == 0) {
+      ices_util_free (stream->name);
+      stream->name = ices_util_strdup ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
+    } else if (strcmp (cur->name, "Genre") == 0) {
+      ices_util_free (stream->genre);
+      stream->genre = ices_util_strdup ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
+    } else if (strcmp (cur->name, "Description") == 0) {
+      ices_util_free (stream->description);
+      stream->description = ices_util_strdup ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
+    } else if (strcmp (cur->name, "URL") == 0) {
+      ices_util_free (stream->url);
+      stream->url = ices_util_strdup ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
+    } else if (strcmp (cur->name, "Mountpoint") == 0) {
+      ices_util_free (stream->mount);
+      stream->mount = ices_util_strdup ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
+    } else if (strcmp (cur->name, "Bitrate") == 0) {
+      stream->bitrate = atoi ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
+    } else if (strcmp (cur->name, "Public") == 0) {
+      stream->ispublic = atoi ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
+    } else if (strcmp (cur->name, "Reencode") == 0) {
+      int res = atoi ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
 
-		} else if (strcmp (cur->name, "Bitrate") == 0) {
-
-			ices_config->bitrate = atoi ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
-
-		} else if (strcmp (cur->name, "Public") == 0) {
-
-			ices_config->ispublic = atoi ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
-
-		} else {
-			ices_log ("Unknown Stream keyword: %s", cur->name);
-		}
+#ifndef HAVE_LIBLAME
+      if (res == 1) {
+	ices_log ("Support for reencoding with liblame was not found. You can't reencode this.");
+	ices_setup_shutdown ();
+      }
+#endif
+			
+      stream->reencode = res;
+    } else if (strcmp (cur->name, "Samplerate") == 0) {
+      stream->out_samplerate = atoi ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
+    } else if (strcmp (cur->name, "Channels") == 0) {
+      stream->out_numchannels = atoi ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
+    } else {
+      ices_log ("Unknown Stream keyword: %s", cur->name);
+    }
 
-		cur = cur->next;
-	}
+    cur = cur->next;
+  }
 }
 
 /* Parse the server specific configuration */
@@ -219,13 +230,6 @@ ices_xml_parse_server_nodes (xmlDocPtr d
 
 			ices_config->host = ices_util_strdup ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
 
-		} else if (strcmp (cur->name, "Mountpoint") == 0) {
-			
-			if (ices_config->mount)
-				ices_util_free (ices_config->mount);
-
-			ices_config->mount = ices_util_strdup ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
-
 		} else if (strcmp (cur->name, "Password") == 0) {
 			
 			if (ices_config->password)
@@ -270,10 +274,6 @@ ices_xml_parse_execution_nodes (xmlDocPt
 			
 			ices_config->verbose = atoi ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
 
-		} else if (strcmp (cur->name, "Samplerate") == 0) {
-			ices_config->out_samplerate = atoi ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
-		} else if (strcmp (cur->name, "Channels") == 0) {
-			ices_config->out_numchannels = atoi ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
 		} else if (strcmp (cur->name, "Base_Directory") == 0) {
 			
 			if (ices_config->base_directory)
@@ -281,18 +281,6 @@ ices_xml_parse_execution_nodes (xmlDocPt
 			
 			ices_config->base_directory = ices_util_strdup ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
 			
-		} else if (strcmp (cur->name, "Reencode") == 0) {
-
-			int res = atoi ((char *)xmlNodeListGetString (doc, ices_xml_get_children_node(cur), 1));
-
-#ifndef HAVE_LIBLAME
-			if (res == 1) {
-				ices_log ("Support for reencoding with liblame was not found. You can't reencode this.");
-				ices_setup_shutdown ();
-			}
-#endif
-			
-			ices_config->reencode = res;
 
 		} else {
 

